#  Install/Load required packages 

libraries <- function(x){
  for( i in x ){
    #  Require returns TRUE invisibly if it was able to load package
    if( ! require( i , character.only = TRUE ) ){
      #  If package was not able to be loaded then re-install
      install.packages( i , dependencies = TRUE )
      #  Load package after installing
      require( i , character.only = TRUE )
    }
  }
}

libraries(c("stats", "dplyr", "ggplot2", "knitr"))

# INPUT: R-code requires three-column matrix where: 1st column corresponds to the average Euclidean distance; 2nd column refers to the experimental endpoint data are available for chemicals from training and validation sets and predicted responses for new screened chemicals, for which experimentally determined values of endpoint are unavailable; 3rd column reflects class membership, where "1" stands for training set (T), "2" means validation set (V), whereas "3" refers to prediction set (P):

#    Distance    Exp/Pred     set
# 1      2.78 -0.4072300     1
# 2      1.44 -1.7713500     1
# 3      1.67  1.0873265     1
# 4      1.94  1.2764568     1
# 5      1.79 -0.9622414     1
# 6      1.65 -0.3950834     2
# 7      1.33 -1.1037483     2
# 8      1.71  0.4289258     2
# 9      1.30 -0.2330070     2
# 10     1.82 -0.2954370     3
# 11     1.66  0.3830070     3
# 12     2.31 -0.7387110     3
# 13     1.99 -0.1550174     3

# Set current file location as the working directory
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

# Importing the input data
Input <- read.csv2("InputData_New.csv")
Data <- data.frame(Input)
Data_T <- filter(Data, set == 1)
Data_V <- filter(Data, set == 2)
Data_P <- filter(Data, set == 3)


# vector of the average Euclidean distance for the training set compounds
Dist_T <- Data_T[, 1]
# vector of the standardized residuals for the training set compounds
Exp_T <- Data_T[, 2]
# vector of the average Euclidean distance for the validation set compounds
Dist_V <- Data_V[, 1]
# vector of the standardized residuals for the validation set compounds
Exp_V <- Data_V[, 2]
# vector of the average Euclidean distance for the prediction set compounds
Dist_P <- Data_P[, 1]
# vector of the standardized residuals for the prediction set compounds
Pred_P <- Data_P[, 2]


# Plotting the distribution of the training data points in the bivariate (i.e. structural domain and response domain) space:
p <- ggplot(Data_T, aes(Dist_T, Exp_T)) +
  geom_point(aes(Dist_T, Exp_T), color = "darkorange3", alpha=0.001, size = 1.5, shape=21) +
  theme_bw() +
  xlab(expression(paste("Average Euclidean distance"))) +
  ylab(expression(paste("Predicted values of log(EC"[50],")"^{-1}))) +
  # X- and Y-axis limits can be manually set up by using the following commands:
  # scale_y_continuous(limits=c(-1.0, 6.0), breaks=seq(-1.0, 6.0, 1.0), labels=c("-1.0", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0")) +
  # scale_x_continuous(limits=c(-1.0, 2.0), breaks=seq(-1.0, 2.0, 1.0), labels=c("-1.0", "0.0", "1.0", "2.0")) +
  # 
  # Labels of the confidence intervals can be additionally printed as follow:
  # annotate(geom="text", x=2.0, y = -3.60, label = "0.99", size = 2.6, color="red") +
  # annotate(geom="text", x=2.0, y = -2.55, label = "0.95", size = 2.6, color="orange") +
  theme(axis.ticks = element_blank(),
        axis.text = element_text(size=8.5),
        axis.title = element_text(size=10.5),
        axis.title.x = element_text(size=10.5),
        axis.ticks.length = unit(0.0, "cm"),
        legend.text=element_text(size=8.5),
        legend.position="top",
        legend.direction="horizontal",
        legend.box = "vertical",
        legend.margin =margin(0,0,0,0),
        plot.margin = unit(c(0.5, 0.5, 0.1, 0.1), "cm"),
        panel.spacing = unit(c(0.01, 0.01, 0.01, 0.01), "cm")) 

# Estimation the reliable prediction space by computing the 95% and 99% confidence interval boundaries derived from the Student's t-distribution 
p + stat_ellipse() 
p + stat_ellipse(type = "t", level = 0.99, linetype = 1, color = "red", size = 0.6, geom = "polygon", fill="goldenrod1", alpha=0.13) +
  stat_ellipse(type = "t", level = 0.95, linetype = 1, color = "orange", size = 0.6, geom = "polygon", fill="forestgreen", alpha=0.14) +
  stat_ellipse(type = "t", level = 0.5, geom = "polygon", fill="seagreen2", alpha=0.005)
theme_bw() 

# Application the defined boundaries (i.e. confidence ellipses) to assess the uncertainty that surrounds the model-based prediction for new screened chemicals, for which experimentally determined values of endpoint are unavailable
p + stat_ellipse () 
p + stat_ellipse(type = "t", level = 0.99, linetype = 1, color = "red", size = 0.6, geom = "polygon", fill="goldenrod1", alpha=0.13) +
  stat_ellipse(type = "t", level = 0.95, linetype = 1, color = "orange", size = 0.6, geom = "polygon", fill="forestgreen", alpha=0.14) +
  stat_ellipse(type = "t", level = 0.5, geom = "polygon", fill="seagreen2", alpha=0.005) +
  geom_point(Data_P, mapping = aes(Dist_P, Pred_P), color = "#372B73", alpha=0.85, size = 2, shape=23) +
  geom_point(aes(x=0.71, y=5.3), color="#372B73", alpha=0.85, size=1.75, shape=23) + 
  annotate(geom="text", x=0.99, y=5.3, label="Prediction set", size=2.5, color="gray8") +
  geom_rect(aes(xmin=0.65, xmax=1.30, ymin=5.1, ymax=5.5), color="gray22", lty=1, lwd=0.05, alpha=0.0) + 
  theme_bw() 


ggsave("AD_DistProb_New.pdf",width = 130, height = 80, unit="mm", dpi=600)